unit utlStrings;
(*
  Unit type (Component, Formless unit, Ancestor form, Application form,...)
  ------------------------------------------------------------------------------
  Formless unit

  Purpose
  ------------------------------------------------------------------------------
  String manipulation

  Notes
  ------------------------------------------------------------------------------
  The tokenizer assumes the token stream to be of the form

  [[whitepace] token [whitepace] { delimiter [whitepace] token }]

  I.e. any number of whitespace characters can surround each token and
  each delimiter, but there can be only one delimiter between tokens.

  Requires ODRegExpUnit by Neil Butterworth.


  History
  ------------------------------------------------------------------------------
  1998-05-00 THk Created


  Copyright
  ------------------------------------------------------------------------------
  (C) 1998 - Thor Kristmundsson


*)

{$BOOLEVAL OFF} // shortcircuit boolean evaluations.

interface
uses classes;

const
  crlf:string=#13+#10;

function strrep(s:string;n:integer):string;
function cat(s:array of string; separator:string=''):string;
function strrpl(s,pat,rpl:string):string;
function strrpls(s:string; pats,rpls:array of string):string;
function rxRpl(text,pat,rpl:string;igncase:boolean):string;
function rxRpls(text:string;pats,rpls:array of string;igncase:boolean):string;
function rxFind(text,pat:string; igncase:boolean):integer;

// tokensizer
type TCharSet=set of char;


procedure frontchop(var s,token:string; valids:TCharSet; maxchop:integer);
function fronttoken(var s,token:string; valids,lim,whitespace:TCharSet):boolean;
function frontstring(var s,token:string; lim,whitespace:TCharSet):boolean;
function frontfloat(var s:string; var value:double; lim,whitespace:TCharSet):boolean;
function frontint(var s:string; var value:integer;
         lim:TCharSet {$ifdef DELPHI4} =[',',#9] {$endif};
         whitespace:TCharSet {$ifdef DELPHI4} =[' '] {$endif}
         ):boolean;

function quote(s:string):string;
function unquote(s:string):string;

procedure StrToProp(obj:TObject; PropName, Value:string);
function PropToStr(obj:TObject; PropName:string):string;

procedure StrToFile(s,fn:string);
function  FileToStr(fn:string):string;

function CheckSum(s:string):integer;
function crypt(text,key:string):string;
function DeCrypt(text,key:string):string;
function EnCrypt(text,key:string):string;


implementation

uses sysutils, utlBase, ODRegExpUnit, typinfo;

function strrep(s:string;n:integer):string;
begin
  result:='';
  while postdec(n)>0 do
    result:=result+s;
end;

function rxFind(text,pat:string; igncase:boolean):integer;
var pattern:string; len:integer;
begin
  pattern := ODRegExpUnit.MakePattern(pat,1);
  result:=ODRegExpUnit.Find(text,1,pattern,len,igncase);
end;

function rxRpl(text,pat,rpl:string;igncase:boolean):string;
var pos,len:integer; pattern:string;
begin
  pattern := ODRegExpUnit.MakePattern(pat,1);
  pos:=ODRegExpUnit.Find(text,1,pattern,len,igncase);
  while pos>0 do
  begin
    assert(len>0);
    delete(text,pos,len);
    insert(rpl,text,pos);
    pos:=ODRegExpUnit.Find(text,pos+length(pat),pattern,len,igncase)
  end;
  result:=text;
end;

function rxRpls(text:string;pats,rpls:array of string;igncase:boolean):string;
var i:integer;
begin
  result:=text;
  assert(high(pats)=high(rpls));
  for i:=low(pats) to high(pats) do
    result:=rxrpl(result,pats[i],rpls[i],igncase);
end;

function cat(s:array of string;separator:string):string;
var i:integer;
begin
  result:='';
  for i:=low(s) to high(s)-1 do
    result:=result+s[i]+separator;
  result:=result+s[high(s)];
end;

// replace each pat in s with a rpl
function strrpl(s,pat,rpl:string):string;
var p:integer;
begin
  p:=pos(pat,s);
  if p>0 then
    result:=copy(s,1,p-1)+rpl+strrpl(copy(s,p+length(pat),length(s)),pat,rpl)
  else
    result:=s;
end;

function strrpls(s:string; pats,rpls:array of string):string;
var i:integer;
begin
  result:=s;
  assert(high(pats)=high(rpls));
  for i:=low(pats) to high(pats) do
    result:=strrpl(result,pats[i],rpls[i]);
end;

procedure frontchop(var s,token:string; valids:TCharSet; maxchop:integer);
var i:integer;
begin
  if maxchop=-1 then
    maxchop:=length(s);
  i:=0;
  while (s<>'') and (s[i+1] in valids) and (i<maxchop) do
    inc(i);
  token:=copy(s,1,i);
  delete(s,1,i);
end;

// "A, B ,C,,D" -> ["A", "B", "C", "", "D"] in succesive calls
function fronttoken(var s,token:string; valids,lim,whitespace:TCharSet):boolean;
var t:string;
begin
  result:=s<>'';
  if not result then
    exit;
  frontchop(s,t,whitespace-lim,-1);     // remove leading spaces
  frontchop(s,token,valids,-1);         // retrieve the meat
  frontchop(s,t,whitespace-lim,-1);     // remove trailing spaces
  frontchop(s,t,lim,1);                 // remove the delimiter
end;

function frontstring(var s,token:string; lim,whitespace:TCharSet):boolean;
begin
  result:=fronttoken(s, token, [#0..#255]-lim-whitespace,lim,whitespace);
end;

function frontfloat(var s:string; var value:double; lim,whitespace:TCharSet):boolean;
var token:string;
begin
  result:=fronttoken(s,token,['0'..'9',DecimalSeparator],lim,whitespace);
  if result then
    value:=strtofloat(token);
end;

function frontint(var s:string; var value:integer; lim,whitespace:TCharSet):boolean;
var token:string;
begin
  result:=fronttoken(s,token,['0'..'9'],lim,whitespace);
  if token='' then
    result:=false;
  if result then
    value:=strtoint(token);
end;

// quote control characters such as CR and LF
function quote(s:string):string;
var i:integer;
begin
  result:='';
  for i:=1 to length(s) do
    if s[i]<' ' then
      result:=result+'^'+char(ord('A')-1+ord(s[i]))
    else
      result:=result+s[i];
end;

// unquote control caracters
function unquote(s:string):string;
var i,len:integer;
begin
  result:='';
  len:=length(s);
  i:=1;
  while i<len do
  begin
    if (s[i]='^') and (s[i+1] in ['A'..'Z']) then
    begin
      result:=result+char(ord(s[i+1])-ord('A')+1);
      inc(i);
    end
    else
      result:=result+s[i];
    inc(i);
  end;
  if i=len then
    result:=result+s[len];
end;

function PropToStr(obj:TObject; PropName:string):string;
var p:PPropInfo;
begin
  p:=GetPropInfo(obj.ClassInfo,PropName);
  assert(p<>nil);
  case p.proptype^.kind of
    tkString,tkLString: result:=getStrProp(obj,p);
    tkInteger,tkEnumeration,tkSet: result:=IntToStr(getOrdProp(obj,p));
    tkFloat: result:=floatToStr(getFloatProp(obj,p));
  end;
end;

procedure StrToProp(obj:TObject; PropName, Value:string);
var p:PPropInfo;
begin
  p:=GetPropInfo(obj.ClassInfo,PropName);
  assert(p<>nil);
  case p.proptype^.kind of
    tkString,tkLString: SetStrProp(obj,p,Value);
    tkInteger,tkEnumeration,tkSet: SetOrdProp(obj,p,StrToInt(Value));
    tkFloat: SetFloatProp(obj,p,StrToFloat(Value));
  end;
end;

procedure StrToFile(s,fn:string);
var lst:TStringList;
begin
   lst:=TStringList.Create;
   lst.Text:=s;
   lst.SaveToFile(fn);
   lst.Free;
end;

function  FileToStr(fn:string):string;
var lst:TStringList;
begin
   lst:=TStringList.Create;
   lst.LoadFromFile(fn);
   result:=lst.Text;
   lst.Free;
end;

function CheckSum(s:string):integer;
var i:integer;
begin
  result:=0;
  for i:=1 to length(s) do
    result:=result+ord(s[i]);
end;

// Weak symmetrical encryption/decryption
function crypt(text,key:string):string;
var i,klen,tlen:integer;
begin
  result:='';
  tlen:=length(text);
  klen:=length(key);
  assert(klen>0);
  for i:=1 to tlen do
    result:=result+char(ord(text[i]) xor ord(key[(i-1) mod klen +1]));
end;

// Weak asymmetrical encryption/decryption routines - untested

function DeCrypt(text,key:string):string;
var i,tlen,klen:integer;
begin
  tlen:=length(text);
  klen:=length(key);
  for i:=1 to tlen do
    result[i]:=chr(ord(text[i])-ord(key[(i-1) mod klen + 1]));
end;

function EnCrypt(text,key:string):string;
var i,tlen,klen:integer;
begin
  tlen:=length(text);
  klen:=length(key);
  for i:=1 to tlen do
    text[i]:=chr(ord(text[i])+ord(key[(i-1) mod klen + 1]));
end;


end.


